#include <oxstd.h> 
#include <oxfloat.h>
#import <maximize>
#import <modelbase>
#include <oxdraw.h>

enum
{
	Y_VAR, X_VAR, Z_VAR, B_VAR, O_VAR, P_VAR, S_VAR, SEL_VAR  	
};
enum // estimation types
{
    M_STATFORC, M_DYNFORC, M_TESTS 
};
enum // model classes					   
{
	MC_COMPARE, MC_COMBINE, MC_REGS 
};

class MulCom : Modelbase			
{
	decl m_iModel;								    /* current model number */
	decl m_iModelClass ;	// Model class
	decl db_mYandForcasts;	// Database with forecasts
	decl m_mObs	;	   		// Ex Post realization of variables to forecast [cT][m_cObs]
	decl m_cObs	;	   		// Number of varibles to forecast [1][1]
	decl m_mP	;	   		// The forecasts [cT][m_cP]
	decl m_cP	;	   		// Number of forecasters [1][1]
	decl m_mB	;	   		// Benchmarks [cT][m_cB]
	decl m_cB	;	   		// Number of Benchmarks [1][1]

	decl m_vY	;	   		// The Dependent var. in Many Regs [cT][1]
	decl m_cY	;	   		// Number of Dependent var. in Many Regs [1][1]
	decl m_mXM	;	   		// The Non-Skrinkage Regressors in Many Regs [cT][m_cXM]
	decl m_cXM	;	   		// Number of Non-Skrinkage Regressors in Many Regs [1][1]
	decl m_mSM	;	   		// The Skrinkage Regressors in Many Regs [cT][m_cXM]
	decl m_cSM	;	   		// Number of Skrinkage Regressors in Many Regs [1][1]

	decl m_sNames;			// Names of the alternative Predictors

	// ManyRegs Settings
	decl m_iTheConst; 		// What to about the constant in regression
	decl m_iRegsToDo;	    // Which Regressions to run
	decl m_iNrToTry;		// Number of regressors to make subsets of
	decl m_mTheSubsToTry;	// Sub sets of regressors to try
	decl m_iSupplySubSets;  // Tell regressor subsets are supplied by the user
	decl m_iHorizon;		// Forecast Horizon

	// ManyRegs Options
	decl m_dSkrinkStrh;		// Skrinkage Strength
	decl m_dSkrinkMeth;		// Skrinkage Meth
	decl m_iMethVardel;		// The Method to use for variable selection 
	decl m_dAlphaVardel;	// The level to use in variable selection 
	decl m_sInfoCrit;		// Infomation criterion used for regression model selection
	decl m_iToSaveForec;	// Whether to save forecasts or not
	decl m_sForecFilename;	// Filename where to save the foecasts
	decl m_sNameForec;		// Name for variable with forecasts
	
	// ManyRegs Variables
	decl m_vForecasts;		// The forecasts
	decl m_vRealized;		// The values of Y which to compare the forecasts

	// Bootstrap Settings
	decl m_cBlocklength;	// Blocklength for blockbootstrap
	decl m_dStatBoot_q;		// q for Stationary Bootstrap
	decl m_cResamples;		// Number of Bootstrap Resamples
	decl m_dMCSalpha;		// Alpha for Model Confidence Set


	// Compare Settings
	decl m_sLossFct;			// The Loss function for SPA, EPA, MCS
	decl m_vTheLossFcts;		// The possible functions for SPA, EPA, MCS
	decl m_sTestStatMCS;		// The Test Statistic for MCS
	decl m_sTestStatSPA;		// The Test Statistic for EPA, MCS
	decl m_vTheTestStatsMCS;	// The Test Statistic for MCS
	decl m_vTheTestStatsSPA;	// The Test Statistic for SPA
	decl m_iToSaveMCSinfo;		// Whether to save forecasts or not
	decl m_sMCSinfoFilename;	// Filename where to save the foecasts
	decl m_sMCSpath;			// Filename where to save the foecasts
	decl m_sBaseDir;			// Filename where to save the foecasts

	// FLAGS
	decl m_iToPrint;		// How much output to print to screen
	decl m_DoSPA;			// To test for SPA or not
	decl m_DoEPA;			// To test for EPA or not
	decl m_DoMCS;			// To compute  MCS or not
	decl m_SelLossTest;			// Whether to do one MCS or all possible MCS's
	decl m_DoFixed;			// To Fixed number of regressors
	decl m_DoAll;			// To All possible regressions
	decl m_iCreateForecDB;	// Create Forecasts database
	decl m_iDeleteForecDB;	// Delete Forecasts database
	decl m_vTests;
	decl m_vOptions;
 	
// Basic functions
	MulCom();
	GetPackageName();
	GetPackageVersion();
	Init_Globals();
	Set_Globals();
	Set_ModelClass(const TheClass); 
	Set_Constant(const Constant);
	Set_Horizon(const horizon); 
	Ch_Forc_Saving(const cre, const del); 
	Set_Loss(const NameLoss);
	Set_TestStatSPA(const NameTest);
	Set_TestStatMCS(const NameTest);
	Set_NameForec(const NameForec);
	Do_SPA(const statboot_q, const cresamples);
	Do_EPA(const cblocklength, const cresamples); 
	Do_MCS(const TheAlpha, const cblocklength, const cresamples);
	Set_Forc_Save_Name(const ForecFilename);
	Set_MCS_Save_Name(const Path, const MCSFilename);
	Set_Reg_Drop(const MethVardel, const AlphaVardel, const InfoCrit);
	Output();
	Pause(const sec);

// Input
	InitData();
    MatrixToString(const name_matrix, const val, const format);	

// Data Handling
	GetDepNames();
	GetObsNames(); 
	GetSkrinkNames(); 
	GetPredNames(); 
	GetBenchNames(); 
	GetParNames();
	virtual GetcT();

// Estimation and Testing

	DoMulCom();		  	 	
	DoManyRegs();		  	 	
	DoFullSelection();
	DoHLN_2003_1();
	DoCombinations();
	SetNforc(const cTforc);
	REGRESSION(const theY, const theXs, const theSs, const Vnames);
	Rest_REGRESSION(const meth, const theY, const theXs, const theSs, const Vnames);
	Which_To_Drop(const theY, const theXs, const theSs);
	Set_TheSubsToTry(const SubsToTry, const SupplySubSets); 
	FindMaxInfoCriterion(const Y,const X, const S);
	GetBetaInference(const Y, const Z, const beta);
	MZ(const HFor, const For);	  
	FEM(const forc, const obs);
	MLEMeth(const par, const parnames, const name, const nbpar, const nobs, const Sk);
	ols_shrink(const Y, const X1, const X2);
	Tests();
    Estimate();
    DoEstimation();

	// OxPack related
	virtual Buffering(const bBufferOn);
	virtual GetMethodLabel();
	virtual GetModelLabel();
	virtual GetModelSettings();
	virtual LoadOptions();
	virtual ReceiveDialog(const sDialog, const asOptions, const aValues);



	virtual ReceiveModel();
	virtual SaveOptions();
	virtual DoOption(const sOpt, const val);
	virtual DoOptionsDlg();
	virtual DoSettingsDlg();
	virtual DoGraphicsDlg();
	virtual ReceiveMenuChoice(const sDialog);
	virtual SendMenu(const sMenu);
	virtual IsCrossSection();
	virtual SendSpecials();
	virtual SendVarStatus();
	virtual SetModelSettings(const aValues);
	virtual BatchCommands();
	virtual BatchMethod(const sMethod);

//	Dialogs();
	ReceiveData();
//	SendDialog(const sDialog);
//	SendMethods();
};

//Database::GetSample()
//{
//	decl it2 = rows(m_mData) - 1;
//	if (m_iFreq == 1)
//		return sprint(ObsYear(0), " - ", ObsYear(it2));
//	else
//		return sprint(ObsYear(0), " (", ObsPeriod(0), ") - ",
//    	    ObsYear(it2), " (", ObsPeriod(it2), ")");
//}
