DeTrend1(const y, const q)
{
/* -- Procedure for producing one-sided detrended versions of a 
      series, using a white-noise + I(2) model.  This produces a 
      very smooth version of the trend component.  The two-sided 
      version of the model produces the HP filter.

      A nice description of what this does is given in 
      Harvey and Jaeger, JAE, July-Sep. 1993, pp. 231-248

      Inputs:
      y = series to be detrended
      q = relative variance of I(2) component
          Note: HP quarterly uses q=.000625 (Kydland Prescott)
                for monthly data a value of q=.00000075
                matches quarterly gain at 50%, 80% and 90% periods
*/
	decl i, vague, f, x, p, xf, h, e, k, xc;
	
	vague=1e+4; // Vague Prior on I(2) component //
	
	// -- Initialize System Matrices -- //
	f      = zeros(2,2);
	f[0][0] = 2;
	f[0][1] = -1;
	f[1][0] = 1;
	x       = zeros(2,1);
	p       = vague*ones(2,2);
	p[0][0] = vague+q;
	xf      = zeros(rows(y),1)*.NaN;
	
	for (i=0;i<rows(y);i++)
	{
	 	x       = f*x;       	// x(t/t-1) //
	 	p       = f*p*f';
	 	p[0][0] += q;    		// p(t/t-1) //

		if (!isnan(y[i]))
		{
	  		h = p[0][0]+1;         // variance of y //
	  		e = y[i]-x[0];        // innovation //
	  		k = p[][0]/h;         // kalman gain //
	  		x = x+k*e;            // x(t/t) //
	  		p = p-(k*p[0][]);     // p(t/t) //
	 	}
	 	xf[i]=x[0];
	}
	
	xc = y-xf;
	
	return(xc~xf);
}
