#include <oxstd.h>
#include <oxfloat.h>
#include <oxdraw.h>
#import  <modelbase>
#import  <packages/PcGive/pcgive_ects>
#import  "Parts/MyRegs"


#include <Procs/detrend1.ox>

main()
{
	decl mX,mGDP,i,temp,useRegs;
    decl QRegs = new MyRegs();       // create object
	decl Freq      = 4;
	decl tFreq     = "Q";
	
	mX = loadmat("data/TaylorRuleData.mat",1);
	mX = reshape(mX,rows(mX)/9,9);
	mX = loadmat("data/trdat.mat",1);
	mX = reshape(mX,rows(mX)/16,16);

	decl st_month = 10*(mX[0][0]-floor(mX[0][0]));
	decl st_year  = floor(mX[0][0]) ;
	decl en_month = 10*(mX[rows(mX)-1][0]-floor(mX[rows(mX)-1][0]));
	decl en_year  = floor(mX[rows(mX)-1][0]) ;	 //	println(st_month~st_year~en_month~en_year) ;

	QRegs.Create(Freq, st_year, 1, en_year, en_month);		//	QRegs.Create(4, 1970, 1, 2006, 4);
	QRegs.Deterministic(-1);
	
	//*****************************************************************************	
	// Add Fed fund rate and Real GDP data

		QRegs.Append( 100*log(1+mX[][1]/100)~diff0(100*log(1+mX[][1]/100),1) ,{"FFR","dFFR"});							
		QRegs.Append( mX[][2]~log(mX[][2]) ,{"GDP","lnGDP"});							

	//*****************************************************************************	
	// Add Inflation data

		decl InflNames = {"GDPD","GDPC","HPCE","CPCE","HCPI","CCPI"};
		QRegs.Append( mX[][3:8] ,InflNames);

		for (i=0;i<sizeof(InflNames);i++){
			temp = QRegs.GetVar(InflNames[i]);
			// 1 Q. Inflation = 400*ln[P(t)/P(t-1)]
			QRegs.Append( Freq*100*diff0(log(temp),1),{InflNames[i]~".I"});
		}
			
	//*****************************************************************************	
	// Add Output Gaps
		temp = QRegs.GetVar("lnGDP"); // print(temp~DeTrend1(temp,.000625));
		QRegs.Append( DeTrend1(temp,.000625)[][0]*100 ,{"GAP_HP"});
		QRegs.Append( diff0(temp,1)[][0]*100 ,{"GAP_Gr"});

		QRegs.SetPrint(FALSE);
		QRegs.Select(Y_VAR, { "lnGDP", 0, 0});
		QRegs.Select(X_VAR, { "Constant", 0, 0, "Trend", 0, 0});
		QRegs.SetSelSample( 1970, 1, 2006, 4);
		QRegs.SetMethod(M_OLS);
		QRegs.Estimate();

		QRegs.Append( QRegs.GetResiduals()*100 ,{"GAP_Tr"}); 		
		QRegs.Append( mX[][10] ,{"GAP_BK"}); 		


	//*****************************************************************************	
	// Add Unemployment
		QRegs.Append( mX[][9] ,{"Unemp"}); 		
		QRegs.Append( mX[][12] ,{"UMP.gap"}); 		

	//*****************************************************************************	
	// Add Real Unit Labor Costs
		QRegs.Append( 100*mX[][14] ,{"Lab.Cost"}); 		

	QRegs.Info();
	//*****************************************************************************	
	// Add Output Interactions

		temp = QRegs.GetVar("lnGDP"); // print(temp~DeTrend1(temp,.000625));
		QRegs.Append( QRegs.GetVar("GDPD.I").*QRegs.GetVar("GAP_HP") ,{"GDPD.I*GAP_HP"});

	//*****************************************************************************	
	// Remove stuff that we don't use

		for (i=0;i<sizeof(InflNames);i++) QRegs.Remove(InflNames[i]);
//		QRegs.Remove("GDP");

		// QRegs.Info();
	QRegs.SaveIn7("In7-data/TRDataBase");	QRegs.SaveXls("In7-data/TRDataBase");


	delete QRegs;

	//*****************************************************************************	
	//*****************************************************************************	
	//*****************************************************************************	
	// taylor Rule regressions
	
	decl TRregs = new MyRegs();
	decl n,sig2,logL;
	
	TRregs.Load("In7-data/TRDataBase.in7");
	TRregs.Deterministic(-1); //	TRregs.Info();
	
	TRregs.Select(Y_VAR, { "FFR", 0, 0});
	useRegs = { "Constant", 0, 0
		,"FFR", 1, 1
		,"GDPD.I", 1, 1
		,"GAP_HP", 1, 1
		,"GDPD.I*GAP_HP", 1, 1
		};
	TRregs.Select(X_VAR, useRegs);
	TRregs.SetSelSample( 1979, 1, 2006, 4);
	TRregs.SetMethod(M_OLS);
	TRregs.Estimate();
	TRregs.TestSummary();

	delete TRregs;


	// taylor Rule regressions
	
	TRregs = new MyRegs();
	
	TRregs.Load("In7-data/TRDataBase.in7");
	TRregs.Deterministic(-1); //	TRregs.Info();
	
	TRregs.Select(Y_VAR, { "FFR", 0, 0});
	useRegs = { "Constant", 0, 0
		,"FFR", 1, 1
		,"GDPD.I", 1, 1
		,"GAP_HP", 1, 1
		,"UMP.gap", 1, 1
		,"Lab.Cost", 1, 1
		};
	TRregs.Select(X_VAR, useRegs);
	TRregs.SetSelSample( 1979, 1, 2006, 4);
	TRregs.SetMethod(M_OLS);
	TRregs.Estimate();
	TRregs.TestSummary();

	delete TRregs;
}


